# include <iomanip>
# include "Page.hh"
# include "Line.hh"   // has-some
# include "Glyph.hh"  // has-some
# include "Image.hh"  // has-

using namespace std;

// C Dependencies (temporary)
# include "otsu.h"     // otsu, thresholding
# include "barcode.h"  // detect_barcode
extern int progress(int p1, int p2);
extern int scan_boxes( pix *p );
extern int add_line_info();
extern int sort_box_func (const void *a, const void *b);
extern int glue_broken_chars( pix *pp );
extern void measure_pitch( Job *job );
extern int find_same_chars( pix *pp);
extern int char_recognition( pix *pp, int mo);
extern int compare_unknown_with_known_chars(pix * pp, int mo);
extern int  try_to_divide_boxes( pix *pp, int mo);
extern int list_insert_spaces( pix *pp, Job *job );
extern int context_correction( Job *job );
extern int  divide_vert_glued_boxes( pix *pp, int mo);

namespace SomeName {
  // ******************************************************************
  // CONSTRUCTORS

  Page::Page(Env* env, const string& file) : Part(NULL, Coord(0,0), Coord(0,0)) {
    // The Coord(0,0) init-list args are dirty, but the fields ARE
    // properly initialized within this constructor!.

    // Every Page maintains its environment
    this->envIs(env);

    // Initialize a Page instance from the given image file.
    this->fileIs(file);

    // FUTURE FIX: For now, we store the same Job instance in Page as
    // we do in Env. However, this will be cleaned up significantly
    // after the basic C++ interface is established.
    Job* job = env->jobRef();
    this->jobIs(job);

    // Eventually, we will be able to read the file into an Image
    // instance using the normal Image constructors. However, since we
    // are relying on gocr code currently, the 'job' field needs to be
    // initialized properly for gocr code to work, and 'job' stores a
    // pixmap for the entire file. If we create an Image using the
    // public constructors, we will end up having two pixmap
    // structures for the entire page. To avoid this, we use a special
    // (temporary) private constructor in Image that shares the
    // pixmap data of job->src.p.

    // Suffix establishes format.  FUTURE FIX: Use magic number instead.
    char* fn = job->src.fname; // fn should equal file
    if ( file != fn ) {
      throw string("Exception: Mismatched filed names in Page::Page");
    }
    bool verbose = job->cfg.verbose;
    if ( file.find(".pcx") != string::npos) {
      readpcx(fn, &job->src.p, verbose);
    } else {
      readpgm(fn, &job->src.p, verbose);
    }
    // Now create the Image using the special constructor.
    Image* image = new Image(verbose);
    this->imageIs(image);

    // Set my topleft and bottomright fields
    this->topleftIs(Coord(0,0));
    this->bottomrightIs(Coord(job->src.p.x-1,job->src.p.y-1));
  }

  // ******************************************************************
  // ACCESSORS
  

  // ******************************************************************
  // METHODS

  // Do everything to convert this page into text.
  bool Page::process() {
    // This is an initial implementation of Page processing. Ambituous
    // individuals can redefine this method in its entirety to obtain
    // a completely new OCR strategy.

    // Partition Page into Glyphs
    bool result = this->partition();
    if ( result ) {
      // Identify Glyphs
      result = this->identify();
    }
    return result;
  }

  bool Page::partition() {
    // Perform analysis
    Job* job = this->jobRef();

    // Set up some local vars
    int  verbose = job->cfg.verbose;
    int  mode    = job->cfg.mode;
    pix* pp      = &(job->src.p);
    bool result  = true;

    // ******************************************************
    // Count Colors (create histogram)
    //  - used to create a upper and lower limit for cs
    //  - cs is the optimum gray value between cs_min and cs_max
    //  - also inverse scans could be detected here later
    if (job->cfg.cs==0) {
      job->cfg.cs=otsu( pp->p,pp->y,pp->x,0,0,pp->x,pp->y, job->cfg.verbose & 1 );
    }
    // renormalize the image and set the normalized threshold value
    job->cfg.cs=thresholding( pp->p,pp->y,pp->x,0,0,pp->x,pp->y, job->cfg.cs );
    if ( verbose ) {
      cerr << "# thresholding new_threshold= %d" << job->cfg.cs << endl;
    }
    progress(5,0); /* progress is only estimated */
    job->tmp.ppo=job->src.p; /* temporarely, removed later */
  
    // ******************************************************
    // Load character database
    if ( mode & 2 ) { load_db(); }
    progress(8,0); /* progress is only estimated */

    // ******************************************************
    // Frame letters
    //  - this is first step for reorganize the PG
    //  - look for letters, put rectangular frames around letters
    //  - letter = connected points near color F
    //  - should be used by dust removing (faster) and line detection!
    //  - 0..cs = black letters, last change = Mai99
    scan_boxes( pp );
    if ( !job->res.numC ) { 
      cerr << "# no boxes found - stopped" << endl;
      if ( verbose & 32 ) { debug_img("out01",job,8); }
      result = false;
      goto EndProcess;
    }
    progress(10,0); /* progress is only estimated */

    // ******************************************************
    // Clean up noise in the image
    //  - iterates over the boxlist.
    remove_dust( job );

    // ******************************************************
    // Border smoothing of characters >7x16
    smooth_borders( job );
    progress(12,0); /* progress is only estimated */

    // ******************************************************
    // Detect barcode
    //  - This would seem to be a rather specialized application?
    //    Most images don't have barcodes in them, why are we always
    //    checking?
    detect_barcode( job );  /* mark barcode */

    // ******************************************************
    // Establishes which boxes in boxlist are pictures.
    //  - for example, any box whose size is 4 times larger in
    //    either direction than the average box size.
    //  - the box is marked as PICTURE if it is a picture.
    detect_pictures( job );

    // ******************************************************
    // Remove boxes marked as pictures from further consideration
    //  - done early so we don't waste time processing them.
    remove_pictures( job );

    // ******************************************************
    // Establish whether the image is already vertically aligned,
    // and if not, what its rotation angle is.
    detect_rotation_angle( job );

    // ******************************************************
    // Establish horizontal lines of characters
    //  - marks up the job->tmp.ppo image for debugging
    detect_text_lines( pp, job->cfg.mode ); 
    progress(20,0); /* progress is only estimated */
    
    // ******************************************************
    add_line_info(/* &(job->res.boxlist) */);
    if(job->cfg.verbose&32) debug_img("out10",job,4+8);
    
    // ******************************************************
    // Detect and divide vertically stacked boxes that touch
    // each other, but which have been established to belong
    // to different lines.
    //  - this must be performed after add_line_info, 
    //    before list_sort
    divide_vert_glued_boxes( pp, job->cfg.mode);
    
    // ******************************************************
    // Characters can be joined together due to serifs.  This
    // attempts to detect such situations and split the box
    // into two (or more).
    //   - makes corrections on pixmap 
    remove_melted_serifs( pp );
    
    // ******************************************************
    // Sort the box list, left to right, top to bottom.
    list_sort(&(job->res.boxlist), sort_box_func);
    
    // ******************************************************
    // Merges boxes that are part of the same character
    //  - a box completely contained within another
    //    box is merged into the surrounding box

    glue_broken_chars( pp );

    // ******************************************************
    //  - removes black borders around bad scans.
    remove_rest_of_dust( );
    
    // ******************************************************
    // Measure pitch
    //  - detect monospaced fonts
    //  - measure overall and per-line pitch
    measure_pitch( job );

    // ******************************************************
    if (mode & 64) { find_same_chars( pp ); }
    progress(30,0); /* progress is only estimated */

    // ######################################################
    // Based on the above SomeName results, we can fully initialize
    // the fields in our C++ classes.
    //  - job->res.boxlist is a collection of Box instances.
    //  - Each Glyph is wrapped around a Box (for now)
    //  - Each Glyph exists within a Word or Line or Region or Page
    //    (usually a Line, for now)
    //  - We may not support Word's to begin with, but when we do,
    //    they will usually be within Lines
    //  - Most documents will contain Lines, but some special-case
    //    "scatterplot" documents in which words are thrown randomly
    //    onto the page won't.
    //  - For now, it is probably easiest to pre-process images
    //    so that there is only one Region per Page (and thus no need
    //    for a Region).  However, advanced support should be able
    //    to detect two-column documents, embedded images, etc.,
    //    in which case each Page will consist of multiple Regions.
    //
    // For now, we iterate over each Box, create an associated Glyph,
    // and accumulate Glyphs into Line instances, and accumulate
    // Lines into the Page.  We do not currently use Word or Region.

    if ( 1 ) {
      List* glist = &(job->res.boxlist);
      // 'start' is a dummy node - we advance to first meaningful one.
      Element* elem = glist->start.next;
      Line*    line = NULL;
      int tbaseline = 0;
      int n         = glist->n;
      int i         = 0;

      // Iterate over the list of boxes
      while ((elem) && (i<n)) {
        // Obtain the Box
        Box* box = (Box*)(elem->data);
        if ( box ) {

          // Create the Glyph around the box
          Glyph* g       = new Glyph(this, box);
          int baseline   = g->baseline();
          if ( baseline != tbaseline ) {
            // Looks like a new line.
            line         = new Line(this, Coord(0,0), Coord(0,0));
            tbaseline    = baseline;
            this->registerPart(line);
          }
          // g->printSummary(cerr,"",i);

          // Add the Glyph to the Line
          line->registerPart(g);
        } else {
          cerr << "Glyph " << i << " has no data" << endl;
        }
        elem = elem->next;
        ++i;
      }

      if ( this->env()->verbose() ) {
        this->printStructure(cerr);
      }
    }

    EndProcess:
    return(result);
  }

  bool Page::identify() {
    // Perform analysis
    Job* job = this->jobRef();

    // Set up some local vars
    int  mode    = job->cfg.mode;
    pix* pp      = &(job->src.p);
    bool result  = true;

    // ******************************************************
    // Perform the actual character recognition!
    char_recognition( pp, job->cfg.mode);
    progress(60,0); /* progress is only estimated */

    // ******************************************************    
    if ( adjust_text_lines( pp, mode ) ) {
      /* correct using chars */
      /* may be, characters/pictures have changed line number */
      list_sort(&(job->res.boxlist), sort_box_func);
      // 2nd recognition call if lines are adjusted
      char_recognition( pp, job->cfg.mode);
    }

    // ******************************************************
    // write out20.pgm - mark lines + boxes
    if(job->cfg.verbose&32) debug_img("out20",job,1+4+8);

    // ******************************************************
    compare_unknown_with_known_chars( pp, job->cfg.mode);
    progress(70,0); /* progress is only estimated */

    // ******************************************************
    try_to_divide_boxes( pp, job->cfg.mode);
    progress(80,0); /* progress is only estimated */

    // ******************************************************
    // list output - for debugging
    if( job->cfg.verbose&6 ) output_list(job);

    // ******************************************************
    list_insert_spaces( pp , job );

    // ******************************************************
    // proof difficult chars Il1 by context view
    if(!(job->cfg.mode&32)) context_correction( job );

    // ******************************************************  
    store_boxtree_lines( job->cfg.mode );
    progress(90,0); /* progress is only estimated */

    // ******************************************************  
    // write out30.pgm
    if( job->cfg.verbose&32 ) debug_img("out30",job,2+4);
    
    progress(100,0); /* progress is only estimated */

    return result;
  }

  void Page::printSummary(ostream& os, const string& indent, int index) const {
    this->Part::printSummary(os,indent,index);
  }

  void Page::output() {
    Job* job = this->jobRef();
    int lc = 0;
    const char* line = getTextLine(lc++);
    int format = job->cfg.out_format;
    while (line) {
      cout << line;
      if (format == HTML ) { cout << "<br/>"; }
      if ( format != XML ) { cout << endl; }
      line = getTextLine(lc++);
    }
    free_textlines();
  }

  void Page::test() {
    Env * env = new Env();
    static const int argc = 5;
    static char* argv[argc] = { "exec", "-i", "test.pnm", "-o", "test.ocr" };
    env->processArgs(argc, argv);
    // Ugh! This is ugly code - stupid global JOB sucks! Relying on
    // job at all sucks, because so much is happening under the hood
    // and isn't obvious. Must fix!
    Job* job = JOB;
    char* fname = job->src.fname;
    cerr << "NOTE: Loading " << fname << endl;
    Page* page = env->currentPage();
    page->partition();

    page->magnifyGlyphs("tmp", 4);
  }
  
};
