
// C++ Dependencies
# include <iostream>
# include <iomanip>
# include <sstream>
# include <string>
# include "Part.hh"  
# include "Page.hh"  // to define 'image()'
# include "Image.hh"

// C dependencies (temporary)
# include "pnm.h"

using namespace std;

namespace SomeName {
  // ******************************************************************
  // CONSTRUCTORS

  Part::Part(Part* parent, const Coord& topleft, const Coord& bottomright) : _topleft(topleft), _bottomright(bottomright) {
    // Deal with parent issues
    if ( parent != NULL ) {
      // This doesn't work as desired because the vtbl hasn't been
      // fully initialized yet. The goal is to have 'this->type()'
      // invoke the subclass definition (and return an integer between
      // 1 and 5), but because it is a virtual method being invoked
      // within the constructor, the local definition is invoked
      // instead.  May need to add a 'validate' method to do this
      // checking.  As long as we don't pass in incorrect parent
      // types this code isn't necessary, so it isn't critical.
      //
      //  FIXTHIS: if ( this->type() <= parent->type() ) { throw string("Attempt to specify a parent image that is smaller than the image itself"); }
    }
    this->parentIs(parent);
  }

  Part::~Part() {
  }

  // ******************************************************************
  // ACCESSORS
  

  // ******************************************************************
  // METHODS

  string Coord::str(int prec) const {
    stringstream s;
    s << "<";
    s << setw(prec) << this->x;
    s << ",";
    s << setw(prec) << this->y;
    s << ">";
    return s.str();
  }

  int Part::type() const {
    // See comments in header file
    throw string("Method 'type' should be redefined by subclasses");
  }

  Image* Part::associatedImage() const { 
    return this->containingPage()->imageRef();
  }

  Page* Part::containingPage() const {
    const Part* parent = this;
    while (parent->parent() != NULL) {
      parent = parent->parent();
    }
    const Page* result = parent->asPage();
    // FUTURE FIX: Can we avoid this const_cast without changing
    // our accessor semantics?
    return const_cast<Page*>(result);
  }

  void Part::registerPart(Part* image) {
    if ( image->type() <= this->type() ) {
      // FUTURE FIX: Exception class
      throw string("Attempt to add a larger image as a child of a smaller image");
    }
    image->parentIs(this);
    this->partsRef().push_back(image);
  }

  void Part::printStructure(ostream& os, const string& indent) const {
    int i = 0;
    const std::list<Part*>& parts = this->parts();
     for (std::list<Part*>::const_iterator it = parts.begin(); it != parts.end(); ++it) {
       Part* image = *it;
       //os << indent << setw(3) << i << ") ";
       image->printSummary(cerr, indent, i);
       ++i;
     }
  }

  void Part::printSummary(ostream& os, const string& indent, int index) const {
    // This default implementation assumes that the subclass has
    // sub-parts, and prints out one. Subclasses may redefine this
    // method, but will usually call this definition then add more
    // info.
    os << ClassName[this->type()];
    if ( index >= 0 ) { os << " #" << index; }
    os << ": ";
    os << endl;
    this->printStructure(os, indent + "  ");
  }

  void Part::magnifyGlyphs(const string& dir, int magnification) const {
    const std::list<Part*>& parts = this->parts();
    if ( parts.size() == 0 ) {
      // We are presumably at a Glyph.  Obtain a magnified Image
      // for the Glyph from the primary Image, and write it to
      // disk.  The directory to write into is specified by 'dir',
      // but the name is dictated by my id().

      // Obtain the magnified sub-image
      Image* image = this->associatedImage();
      Image gim(image, this->topleft(), this->bottomright(), magnification);

      // FUTURE FIX: Create the directory if it doesn't exist!
      // ...

      // Create the basefile name to write to.
      string base = dir;
      base += "/";
      base += this->id();

      // Now write gim.
      gim.writeGrey(base);
      
    } else {
      // Recurse on each part.
      for (std::list<Part*>::const_iterator it = parts.begin(); it != parts.end(); ++it) {
	(*it)->magnifyGlyphs(dir, magnification);
      }
    }
  }

  int Part::findIndex(const Part* image) const {
    const std::list<Part*>& parts = this->parts();
    int result = -1;
    int i = 0;
    for (std::list<Part*>::const_iterator it = parts.begin(); it != parts.end(); ++it) {
      if ( image == *it ) {
        result = i;
        break;
      }
      ++i;
    }
    return result;
  }

  string Part::id() const {
    // NOTE: This is not an efficient method - 'id' is only
    // meant to be used for debugging purposes, not core functionality.
    stringstream s;

    s << "id";
    const Part* image = this;
    const Part* parent = image->parent();
    while (parent != NULL) {
      int i = parent->findIndex(image);
      if ( i == -1 ) {
        // This should never happen - implies that the parent/parts
        // tree-structure has been corrupted.
        throw string("InvalidTreeException: Failed to find an image in the parts-list of its parent");
      } else {
        // We have found the index of 'image' within its parent, and
        // add that index to 's'
        s << "_" << setfill('0') << setw(3) << i;
      }
      image = parent;
      parent = image->parent();
    }
    return s.str();
  }

  char* Part::ClassName[6] = {
    "Part",
    "Page",
    "Region",
    "Line",
    "Word",
    "Glyph",
  };

  void Part::test() {
    cerr << "ERROR: Not unit test for Part - FIX THIS!" << endl;
  }

};
