# ifndef Part_hh
# define Part_hh Part_hh

// ####################################################################
// # <name/>: An Extensible Optical Character Recognition Framework   #
// #                                                                  #
// # Copyright: GNU Public License                                    #
// #                                                                  #
// # Overview:                                                        #
// #  - <name/> is a collection of C++ classes defining an OCR engine #
// #  - <name/> is designed to allow customization at all levels      #
// #    and to encourage individuals to contribute incremental        #
// #    improvements in algorithms.                                   #
// #  - Overall design discussions can be found at                    #
// #      ...                                                         #
// #  - <name/> is built on top of other open-source OCR programs:    #
// #     - SomeName, by Joerg Schulenburg                             #
// #     - Ocrad, the GNU OCR                                         #
// #                                                                  #
// # Authors:                                                         #
// #    Wade Holst  (overall design)                                  #
// ####################################################################


// C++ Dependencies
# include "Root.hh"  // parent
# include <list>     // field 'parts'
# include <iostream> // cerr
# include "Coord.hh" // Coord

// Declaration
namespace SomeName {
  // Fixed subclass forward declarations. Needed for the subclass
  // typecasting methods 'asPage', 'asGlyph', etc.  I know its dirty,
  // but it is also useful.
  class Page;  
  class Region;
  class Line;  
  class Word;  
  class Glyph; 
  class Image;

  // ###################################################################
  /// \class Part
  ///
  /// \brief Abstract superclass of a collection of classes representing
  /// a part-whole decomposition of a graphic image into smaller and
  /// smaller semantic units.
  ///
  /// The Part class is at the root of a critical hierarchy of
  /// subclasses. Certain subclasses (Page and Glyph) reprsents the most
  /// important classes in the implementation, and much of their
  /// functionality is inherited from Part. Other subclasses (Region,
  /// Line and Word) provide "refinements" but aren't strictly necessary
  /// to the proper execution of the code.
  ///
  /// An Part consists of state and functionality supporting a
  /// hierarchial decomposition of larger parts into a collection of
  /// smaller Parts contained within their "parent" part. The subclasses
  /// of Part have a conceptual "size order", and a tree structure links
  /// Parts to their parents and to their contained sub-images. Pages
  /// are the biggest, Regions are contained within Pages, Lines are
  /// contained within Regions or Pages, Words are contained within
  /// Lines, Regions, or Pages, and Glyphs are contained within Words,
  /// Lines, Regions or Pages.
  ///
  /// Implementation subtleties:
  /// --------------------------
  ///
  /// Since Parts are part of a part-whole decomposition into smaller
  /// and smaller images, the manner in which this decomposition is
  /// implemented can have significant time and space efficiency
  /// ramifications.
  ///
  /// One strategy would be to have each Part subclass maintain a local
  /// copy of that portion of the overall image to which it applies.
  /// Fields 'height' and 'width' would establish the pixel dimensions,
  /// and a 'data' field could store the actual pixel information. For
  /// example, each Glyph could maintain a copy of that portion of the
  /// Page represented by the Glyph.
  ///
  /// The problem with the above naive implementation is that it incurs
  /// more and more memory the more sub-divided an Part becomes.
  /// Although it is common for Pages to contain just a collection of
  /// Glyphs, it is also possible for a Page to contain Regions that
  /// contain Lines that contain Words that contain Glyphs (which might
  /// contain other Glyphs). In such a situation, the pixel data
  /// representing an individual Glyph would be copied (and maintained
  /// in memory) up to 5 times (stored in a Glyph, stored in a Word,
  /// stored in a Line, stored in a Region, and stored in the Page.
  ///
  /// The above memory impact can be avoided by taking a different
  /// approach. Instead of each Part maintaining a separate copy of its
  /// portion of an image, we note that the largest possible Part is a
  /// Page, and all other subclasses of Page are conceptually contained
  /// within a Page. If each Part has access to its containing Page,
  /// then we could replaced the 'data' field with a <x,y> coordinate
  /// identifying the topleft pixel within the Page at which the
  /// sub-image begins. In addition, we could maintain either a
  /// <width,height> pair, or another <x,y> pair representing the bottom
  /// right corner (either one will allow computation of the other). By
  /// maintaining topleft relative to the Page, we do not need to
  /// maintain multiple copies of the data (which is what took up most
  /// of the memory in the first variant above).
  ///
  /// Because this second strategy significantly reduces memory costs,
  /// it is the strategy currently used. However, the interface is
  /// separate from the implementation, and we can change the
  /// implementation as desired if it becomes necessary.
  ///
  /// An Part has the following fields:
  ///   parent       --> the parent Part
  ///   parts        --> the list of sub parts
  ///   topleft      --> the <x,y> pixel coordinate within the Page image to
  ///                    which this image belongs of the top-left corner
  ///                    of this sub image.  <x,y> is <0,0> for Page
  ///                    instances.
  ///   bottomright  --> the <x,y> pixel coordinate within the Page image to
  ///                    which this image belongs of the bottom-right corner
  ///                    of this sub image. 
  ///
  /// An Part can establish its Page by traversing its 'parent' links
  /// until one is found that has no parent (Page instances have a NULL
  /// parent).
  // ###################################################################

  class Part : public Root {
  public:
    // Types

    // Constructors/Destructors

    /// Each Part specifies its parent.
    ///
    /// Checks are performed to ensure that the parent specified
    /// can be a parent of the image in question.  See the 'type'
    /// method and 'parent' field for more details.
    ///
    /// It is possible for parent to be NULL. When the reliance on C
    /// code is removed, this should only occur for Page instances
    /// (which do not have an Part parent), but currently the C code
    /// creates by-value Glyphs using a default constructor, so we
    /// must allow them to have NULL parents as well. Note that the
    /// Page constructor always passes a NULL pointer to its parent
    /// constructor, and that Glyph has a default value of NULL, while
    /// the other subclasses require a real image.
    Part(Part* parent, const Coord& topleft, const Coord& size);
    virtual ~Part();
    
    // Accessors
    inline const Coord &   topleft() const { return this->_topleft; }
    inline const Coord &   bottomright() const { return this->_bottomright; }
    inline const std::list<Part*>& parts() const { return this->_parts; }
    inline const Part*    parent() const { return this->_parent; }
    
    // Input/Output
    
    // Miscellaneous

    /// Returns an integer establishing how "small" this image type
    /// is, relative to other image types. It has nothing to do with
    /// width or height, but instead with the conceptual size of the
    /// type itself. All instances of a particular subtype will always
    /// return the same value. The code is designed so that Glyph
    /// returns a larger number than Word, which is larger than Line,
    /// which is larger than Region, which is larger than Page. This
    /// allows us to perform some sanity checks on hierarchial
    /// decompositions to ensure that we don't make silly structures
    /// in which Lines have Glyphs as parents, etc.
    ///
    /// FUTURE FIX: This method should be pure-virtual, but making it
    /// pure-virtual causes failure during compilation due to code in
    /// the constructor. Must determine why the compilation error is
    /// occuring and resolve it.
    virtual int type() const;

    /// Add an Part to the set of parts within myself.
    void registerPart(Part* part);
    Part* firstPart() const { return ( this->parts().empty() ) ? NULL : this->parts().front(); }
    Part* lastPart() const  { return ( this->parts().empty() ) ? NULL : this->parts().back();  }

    Coord size() const { return Coord( this->bottomright().x - this->topleft().x + 1, this->bottomright().y - this->topleft().y + 1 ); }

    /// Returns the Page within which this Part resides.
    ///
    /// If the Part is a Page, returns itself.  Otherwise,
    /// the Part is a sub-region of a Page (and the Page of which
    /// it is a sub-region is returned).
    ///
    /// This method is important because the 'topleft' and
    /// 'bottomright' fields of an Part describe coordinates within
    /// the Image associated with Page. That is, Part subclasses other
    /// than Page do not maintain their own local Images, but instead
    /// maintain information establishing what sub-region of the Page
    /// Image they apply to. Doing so allows us to avoid maintaining
    /// relative coordinates and redundant copies of parts.
    Page* containingPage() const;

    /// The Image that this Part is a sub-region of.
    ///
    /// Every subclass of Part stores topleft and bottomright
    /// coordinates relative to an Image stored within a Page. This
    /// method returns the Image.
    ///
    /// Not named 'image' to avoid confusing with the accessors in
    /// Page for field 'image'.
    virtual Image* associatedImage() const;

    /// Although there are good reasons for superclasses not knowing
    /// about subclasses, I find the convenience of these downcasting
    /// methods too useful to avoid. Redefinitions are provided in
    /// subclasses for the relevant method (Page::asPage() returns
    /// 'this', Glyph::asGlyph returns 'this', etc.)
    virtual Page*   asPage()   { return NULL; }
    virtual Region* asRegion() { return NULL; }
    virtual Line*   asLine()   { return NULL; }
    virtual Word*   asWord()   { return NULL; }
    virtual Glyph*  asGlyph()  { return NULL; }

    virtual const Page*   asPage()   const { return NULL; }
    virtual const Region* asRegion() const { return NULL; }
    virtual const Line*   asLine()   const { return NULL; }
    virtual const Word*   asWord()   const { return NULL; }
    virtual const Glyph*  asGlyph()  const { return NULL; }

    // **********************************************************
    // Debugging methods

    /// Print out the containment hierarchy for this image.
    void printStructure(std::ostream& os = std::cerr, const std::string& indent = "") const;
    virtual void printSummary(std::ostream& os = std::cerr, const std::string& indent = "", int index = -1) const; 

    /// Create .pgm files for every Glyph within this Part
    void magnifyGlyphs(const std::string& dir, int magnification) const;

    /// Returns a unique string identifying this image among all
    /// others.
    /// 
    /// Consists of 'id', followed by integers separated by
    /// underscores. Each integer identifies a position within a
    /// container (Region, Line, Word, Glyph). The result is designed
    /// to be useful in filenames.
    ///
    // NOTE: This is not an efficient method - it is only meant to be
    // used for debugging purposes, not core functionality.
    virtual std::string id() const;

    /// Returns the index of 'image' within this Part's list of
    /// subparts.
    ///
    /// Returns -1 if the image is not an immediate sub-image of
    /// this image.
    int findIndex(const Part* image) const;

    /// Unit testing method.
    ///
    /// This static method should create instances of the class (and
    /// instances of any other class necessary) and perform tests to
    /// ensure that all methods within the class are working as
    /// expected.
    static void test();
    
  protected:
    // Accessors
    inline void            topleftIs(const Coord & topleft) { this->_topleft = topleft; }
    inline Coord &         topleftRef() { return this->_topleft; }
    inline void            bottomrightIs(const Coord & bottomright) { this->_bottomright = bottomright; }
    inline Coord &         bottomrightRef() { return this->_bottomright; }
    inline void            partsIs(const std::list<Part*> & parts) { this->_parts = parts; }
    inline void            parentIs(Part* parent) { this->_parent = parent; }
    inline Part* &        parentRef() { return this->_parent; }
    
    // Methods 

    // Fields
    static char* ClassName[6];
    
  private: 
    // Accessors
    inline std::list<Part*> & partsRef() { return this->_parts; }
    
    // Methods 
    
    // *******************
    // State

    /// The coordinate (in pixels) of the topleft corner of this image
    /// within its containing Page (relative to the topleft of the
    /// Page, with pixel indices starting at 0).
    Coord           _topleft;

    /// The coordinate (in pixels) of the bottom right corner of this
    /// image within its containing Page (relative to the topleft of
    /// the Page, with pixel indices starting at 0).
    Coord           _bottomright;

    /// The parent Part of this Part
    ///
    /// There is a hierarchial decomposition of parts associated
    /// with this code, and this field maintains a link to the parent
    /// in this decomposition.  For example, each Glyph belongs
    /// to a Page, but not necessarily directly.  It could belong
    /// to a Word that belongs to a Line that belongs to a Region
    /// that belongs to a Page.  Or it could belong directly to
    /// a Page, or any variation in between these two.
    ///
    /// However, an image should never have as a parent another Part
    /// that is conceptually "smaller" than it.  For example, a Word
    /// should not have a Glyph parent, a Line should not have
    /// a Word or Glyph parent, etc.  This is enforced at runtime
    /// within the constructor by ensuring that the 'type' of
    /// the child is greater than the 'type' of the parent.
    Part*          _parent;

    /// The collection of sub-Parts within this Part
    ///
    /// A Page contains Regions and/or Lines and/or Words and Glyphs,
    /// a Region contains Lines and/or Words and Glyphs, a Line
    /// contains Words and/or Glyphs, Words contain Glyphs, and Glyphs
    /// may contain other Glyphs.
    ///
    /// Although compile-time type-checking could have been provided
    /// by having individual subclasses define this field with a
    /// specific type, that approach would have precluded us from
    /// allowing a Page to contain Regions OR Lines OR Words OR Glyphs
    /// (we would have had to pick one of them, or introduce a
    /// superclass for some subset of them). We can, however, provide
    /// run-time typechecking to ensure that we do not try to add a
    /// Line as a child of a Word, etc. See the 'registerPart' method
    /// for more.
    ///
    /// Note that although some subclasses could benefit from having a
    /// vector to improve random access, it is more likely that we
    /// will need to insert/delete sub-parts than it is that we we
    /// need random access. We can use the list iterators for
    /// traversal, and see the 'firstPart' and 'lastPart' methods
    /// for more.
    std::list<Part*> _parts;
  };
};

# endif // Part_hh

