# ifndef Root_hh
# define Root_hh Root_hh

/// \mainpage SomeName
/// 
/// This document provides an overall design discussion of SomeName.
/// explaining and justifying the architecture, providing an overview of
/// the purpose of each class, the most important parts of the public
/// interface of each class, etc. Much more detailed documentation is
/// available on a per-class basis within the classes themselves, and via
/// the doxygen pages in 'docs/'.
/// 
/// <hr>
/// \section hierarchy Class Hierarchy
///
/// \verbatim
//       
//                                            Root
//                                              |
//       .---------------------------------------^-----------.
//       |                                       |           |
//       |                                      Part       Env
//       |                                       |   
//       |               .-----------------------^-----------------------.
//       |               |            |          |          |            |    
//       Image <-------- Page        Region       Line      Word         Glyph
//            
//\endverbatim
/// 
/// 
/// <b>Root</b>: Abstract superclass of all classes in the SomeName
/// framework.
/// 
/// <b>Env</b> : A class providing a public interface to other
/// programs.
/// 
/// <b>Image</b> : A collection of pixels. In-memory representation of
/// a file in a known graphics format.
/// 
/// <b>Part</b> : Abstract superclass of a collection of classes
/// representing a part-whole decomposition of a graphic image into
/// smaller and smaller semantic units.
/// 
/// <b>Page</b> : An Image and associated meta-data representing an
/// entire page of to-be-scanned data. May be sub-divided into
/// Regions, Lines, Words and/or Glyphs, although only Glyphs are
/// crucial.
/// 
/// <b>Region</b> : A sub-region of a Page, usually used to represent
/// the graphical area for a single column in a multi-column image.
/// May be sub-divided into Lines, Words, and/or Glyphs. This class is
/// not currently prioritized and may be deleted if deemed
/// unnecessary.
/// 
/// <b>Line</b> : A sub-region of a Page (or Region) consisting
/// entirely of Glyphs. My be sub-divided into Words that contain
/// Glyphs, but may often contain only Glyphs.
/// 
/// <b>Word</b> : A sub-region of a Line consisting entirely of
/// Glyphs, separated from other Words within the Line by more
/// horizontal space than
/// 
/// <b>Glyph</b> : A sub-region of a Page (or Region or Line or Word)
/// representing a single to-be-identified character.
/// 
/// <hr>
/// \section terminology Terminology
/// 
///   - horizontal: the direction that aligns glyphs within a line.
///     Normally, this is just the x direction, but if the image is
///     rotated, then 'horizontal' refers to the x direction after
///     this rotation has been compensated for.
/// 
///   - vertical: the direction orthogonal to horizontal.
/// 
///   - line partitioning: The identification of the regions within a
///     Page representing individual Lines of horizontally adjacent
///     Glyphs. Line partitioning may occur before or after Glyph
///     partitioning, depending on the overall partitioning strategy
///     employed.
/// 
///   - glyph partitioning: The identification of the regions withiin
///     a Page representing individual Glyphs. Glyph partitioning is
///     not the same as Glyph identification, but instead preceeds it.
/// 
///   - glyph identification: The process of converting a graphic
///     image representing a character into a unicode value. The heart
///     of any OCR program. Its efficacy depends not only on its
///     internal strategies, but also the accuracy of Glyph
///     partitioning. If a region is identified as a Glyph, but is
///     instead only part of a glyph, or multiple glyphs, the accuracy
///     will naturally decline.
/// 
/// <hr>
/// \section guidelines Programming Guidelines
/// 
/// Readability, code-reuse, abstraction, modularity, encapsulation and
/// information hiding aren't just buzzwords, they are critical to smooth,
/// ongoing evolution of code. And they don't occur by magic. Here are
/// some guidelines to follow when working on this project.
/// 
///  - All fields are defined in private scope and are preceeded by an 
///    underscore.  Accessors are defined for each field.  See the
///    more detailed discussion on 'Accessors' below.
///     - getters can be public, but should ALWAYS have read-only 
///       semantics, and should be moved out of 'public' scope if the
///       field isn't critical to the public interface.
///     - setters should never be at public scope
/// 
///  - Classes should be fully initialized by their constructors. Partial
///    initialization requires clients of the class to remember to perform
///    any additional initialization before using the class normally, and
///    completely defeats the point of having an initializer to begin
///    with. Furthermore, it requires setter methods to either be made
///    public (bad) or requires other classes to be added as friends (to
///    be avoided when possible). Full initialization within constructors
///    implies that classes should NOT have default constructors unless
///    the default values assigned to fields are not only intuitive and
///    meaningful, but also useable as-is.
/// 
///  - Every class is documented. Every method is documented. Every field
///    is documented. Use '///' so that the class/method/field
///    documentation is available to doxygen. The UML diagram(s) are kept
///    up-to-date. Overall changes in design are kept up-to-date.
/// 
///  - Every class should have a static 'test()' method that acts as a
///    unit test. This method should create one or more instances of the
///    class (and any other classes needed to get the class to work) and
///    test the public interface methods, checking return results/output
///    against expected results to verify that it is working as expected.
/// 
///  - Avoid symbol conflicts. All classes are placed in the SomeName
///    namespace. No global variables. Do NOT place 'using namespace std'
///    statements in header files. Yes, I know it allows you to avoid
///    putting 'std::' in front of all sorts of types, but putting such a
///    statement in header files forces third-party code to fully-qualify
///    class names that conflict with those in 'std'. Using 'using
///    namespace std' in source files is completely ok.
/// 
///  - Avoid assuming there will only be one instances of any particular
///    class. Support for multi-threading/parallel/distributed algorithms
///    suggests that even classes that seem like Singletons may not be.
/// 
///  - Write code in a thread-safe manner.  Avoid static variables were
///    possible.
/// 
/// <hr>
/// \subsection accessors Accessors
/// 
/// Suppose we have a field called 'name', of type 'Type' (for example,
/// 'Type' might be 'int' or 'A*'). One way to define it would be:
/// 
/// \code
//   class A {
//     public:
//       Type name;
//   }  
//\endcode
/// 
/// And then an individual would read/write the field using:
///
/// \code
//   A* a = new A;
//   a->name = 10;
//   cout << "name = " << a->name << endl;
//\endcode
/// 
/// A public field is a terrible implementation decision because:
///   1) All code, everywhere, is allowed to modify (and thus potentially
///      corrupt) its value.  There is no "safety" available and
///      modification-at-a-distance is difficult to debug.
///   2) If, during the evolution of the code, the type of the field
///      (or the existence of the field) comes into question, changes
///      are both difficult, and guaranteed to break any third-party
///      code that relies on this field.
/// 
/// However, using a simple strategy can address both of these problems
/// and add some additional capability. And in C++, this comes at
/// absolutely no runtime performance cost, as we'll see below.
/// 
/// Instead of defining our field as shown above, we do the following:
/// 
/// \code
//   class A {
//     public:
//        inline const Type& name() const { return this->_name; }
//
//     protected:
//        inline void nameIs(Type val) { this->_name = val; }
//        inline Type& nameRef() { return this->_name; }
//
//     private:
//       Type _name;
//   }
//\endcode
/// 
/// In the above, we have moved the field definition itself into private
/// scope, and places an underscore in front of it.  We have also added
/// three tiny inlined methods providing get/set access to the field.
///
/// <hr>
/// \subsubsection features Features worth noting:
/// 
///   - The 'name()' method returns a const version of the underlying
///     field. It allows 'read-only' access to the field, but does not
///     allow it to be modified. It is often at public scope, but should
///     be placed in protected scope if the field is not part of the
///     public interface. Obviously, the fewer field accessors in the
///     public interface, the more separation between implementation and
///     interface.
/// 
///   - The 'nameIs()' method sets the value of the field to its single
///     argument. This method should almost never be placed at public
///     scope. Classes should be designed so that they, are a small set of
///     tightly-coupled friend classes, are responsible for modifying
///     themselves. In this way, corruption of state is much easier to
///     identify, because only a small number of classes can be
///     responsible, and ultimately the change had to have occured via
///     this setter method.
/// 
///   - The 'nameRef()' method has exactly the same code block as
///     'name()', but its return value and non-const status make it
///     much more powerful.  It too should almost never be placed at
///     public scope, as it provides a second means of modifying the
///     field.  For example, if Type is 'int', we can write:
/// \code
//        a->nameRef() = 10;
//\endcode
///     because 'nameRef()' returns a reference to the 'int' field. This
///     accessor is useful because anything we could do with the
///     underlying field 'a->_name', we can also do with 'a->nameRef()'.
///     For example:
/// \code
//         cout << a->_name << endl;
//         a->_name += 10;
//         ++a->_name;
//\endcode
///     is the same as
/// \code
//         cout << a->nameRef() << endl;
//         a->nameRef() += 10;
//         ++a->nameRef();
//\endcode
///     except that using 'nameRef' provides for some separation of
///     interface and implementation.  Note that since 'nameRef' is
///     inline, the above two sets of lines really are identical
///     after compilation.
/// 
/// <hr>
/// \subsubsection features2 Additional Features
/// 
///   - The type of the argument in 'nameIs()' can sometimes be made
///     'const', but when it is a pointer type, it cannot be made 'const'
///     unless the underlying field itself is 'const'.
/// 
///   - Although many C++ programmers avoid using 'const', constness
///     can have a significant positive impact on code quality.  Rather
///     than solving 'const' issues by not using 'const', it takes only
///     a little effort to understand the basic principles.  Remember that:
/// 
///       - If a variable is 'const', it cannot be modified. That is, if
///         it appears on the right-hand side of an assignment statement,
///         the compiler will generate an error.
///   
///       - 'const'ness is contagious - if one variable is 'const', it
///         often forces other variables or methods to also be 'const',
///         which may in turn force other variables or methods to be
///         'const', etc. etc. Because of this, the strategy of "I'll get
///         my code working without 'const' first, then add 'const'
///         semantics in later" is a terrible idea, because it is
///         difficult to "add in 'const'" incrementally - making one
///         variable/method 'const' requires many others to be made const
///         before the code will compile again.  Because of this, it is
///         much better to always make EVERYTHING (variables and methods)
///         'const' by default, at least conceptually, then remove
///         'const' when it is established that the variable/method in
///         question cannot be 'const' and still retain the desired
///         semantics.
/// 
///       - If a variable is 'const', then only 'const' methods can
///         be invoked with that variable as a receiver.  For example:
/// \code
//             const Person p;
//             p.somefunc();
// \endcode
///         Only if 'somefunc' is marked as 'const' in its definition:
/// \code
//             void somefunc() const { ... }
// \endcode
///         will the above 'p.somefunc();' invocation be allowed by
///         the compiler.
///       
///       - A non-static method f() defined within a class A has a
///         special variable called 'this'.  Since it is a variable,
///         it must have a type.  The type of 'this', within class A,
///         is ALWAYS either 'A* const' or 'const A*const'.  Which
///         one it is depends on whether the method f() is 'const'
///         or not.  
/// 
///          - A method is 'const' if the keyword 'const' appears
///            AFTER its parameter list.
/// 
///          - If 'this' is of type 'A* const', it means that the 
///            space for the pointer itself cannot be changed, so the
///            statement
/// \code
//                this = a;
// \endcode
///            is not allowed.  Since the 'this' variable is ALWAYS
///            'A* const' or 'const A*const', one is never allowed to
///            modify 'this'.  However, a variable being '*const'
///            does not affect the ability to modify what the variable
///            points to, so 
/// \code
//                this->some_field = 10;
// \endcode
///            isn't disallowed.
/// 
///          - If 'this' is of type 'const A* const', it means that
///            everything above for 'A* const' holds, AND that we
///            cannot modify what the variable points to, so
/// \code
//                this->some_field = 10;
// \endcode
///            becomes a compile-time error.
/// 

namespace SomeName {
  // ###################################################################
  /// \class Root
  ///
  /// \brief The abstract superclass of all classes in the SomeName
  /// namespace.
  ///
  /// Functionality defined here is available in every SomeName class,
  /// obviously. Primarily debugging and other low-level functionality.
  // ###################################################################

  class Root {
  public:
    // Constructors/Destructors
    Root();
    
    // Accessors
    
    // Input/Output
    
    // Miscellaneous
    
  protected:
    // Accessors
    
    // Methods 
    
  private: 
    // Accessors
    
    // Methods 
    
    // State
  };
};

# endif // Root_hh

